@@ -22,10 +22,12 @@
 #include <gtkmm/scrolledwindow.h>
 #include <gtkmm/treeview.h>
 #include <gtkmm/treestore.h>
+#include <gtkmm/treemodelsort.h>
 #include "buffer_def.hpp"
 #include "document_settings.hpp"
 #include "togglewindow.hpp"
 #include "header.hpp"
+#include "folder.hpp"
 
 namespace Gobby
 {
@@ -38,6 +40,7 @@ public:
 	DocumentList(Gtk::Window& parent,
 	             DocumentSettings& settings,
 	             Header& header,
+		     Folder& folder,
 		     const Preferences& preferences,
 		     Config::ParentEntry& config_entry);
 
@@ -57,16 +60,20 @@ protected:
 
 	void on_subscribe();
 	void on_selection_changed();
-	bool on_tree_button_press(GdkEventButton* event);
+	void on_row_activated(const Gtk::TreePath& path,
+	                      Gtk::TreeViewColumn* column);
 
 	LocalBuffer* m_buffer;
 	DocumentSettings& m_settings;
 
+	Folder& m_folder;
+
 	Gtk::VBox m_mainbox;
 	Gtk::Button m_btn_subscribe;
 
 	Gtk::ScrolledWindow m_scrolled_wnd;
 	Gtk::TreeView m_tree_view;
+	Glib::RefPtr<Gtk::TreeModelSort> m_sorted;
 
 	Gtk::TreeViewColumn m_view_col;
 };
