@@ -17,6 +17,7 @@
  */
 
 #include <gtkmm/stock.h>
+#include <gtkmm/treemodelsort.h>
 #include "common.hpp"
 #include "documentlist.hpp"
 
@@ -35,6 +36,7 @@ namespace
 Gobby::DocumentList::DocumentList(Gtk::Window& parent,
                                   DocumentSettings& settings,
                                   Header& header,
+                                  Folder& folder,
                                   const Preferences& preferences,
 				  Config::ParentEntry& config_entry):
 	ToggleWindow(
@@ -45,6 +47,7 @@ Gobby::DocumentList::DocumentList(Gtk::Window& parent,
 	),
 	m_buffer(NULL),
 	m_settings(settings),
+	m_folder(folder),
 	m_btn_subscribe(_("Subscribe") )
 {
 	m_view_col.pack_start(settings.columns.icon, false);
@@ -70,13 +73,14 @@ Gobby::DocumentList::DocumentList(Gtk::Window& parent,
 		settings.columns.color
 	);
 
-	m_view_col.set_sort_column(settings.columns.title);
-
 	m_tree_view.add_events(Gdk::BUTTON_PRESS_MASK);
-	m_tree_view.signal_button_press_event().connect(
-		sigc::mem_fun(*this, &DocumentList::on_tree_button_press) );
+	m_tree_view.signal_row_activated().connect(
+		sigc::mem_fun(*this, &DocumentList::on_row_activated) );
 
-	m_tree_view.set_model(settings.get_list() );
+	m_sorted = Gtk::TreeModelSort::create(settings.get_list() );
+
+	m_sorted->set_sort_column(settings.columns.title, Gtk::SORT_ASCENDING);
+	m_tree_view.set_model(m_sorted);
 	m_tree_view.append_column(m_view_col);
 	m_tree_view.set_headers_visible(false);
 
@@ -111,6 +115,10 @@ void Gobby::DocumentList::obby_start(LocalBuffer& buf)
 	// Clear old data
 	m_scrolled_wnd.set_sensitive(true);
 	m_buffer = &buf;
+
+	// This is necessary because otherwise on_selection_changed()
+	// might be called when the document info is already destroyed.
+	m_tree_view.get_selection()->unselect_all();
 }
 
 void Gobby::DocumentList::obby_end()
@@ -173,8 +181,10 @@ void Gobby::DocumentList::on_subscribe()
 	    iter != selected_entries.end();
 	    ++ iter)
 	{
+		Gtk::TreePath unsorted_path =
+			m_sorted->convert_path_to_child_path(*iter);
 		Gtk::TreeIter tree_iter =
-			m_settings.get_list()->get_iter(*iter);
+			m_settings.get_list()->get_iter(unsorted_path);
 
 		LocalDocumentInfo* info =
 			(*tree_iter)[m_settings.columns.info];
@@ -197,8 +207,10 @@ void Gobby::DocumentList::on_selection_changed()
 	    iter != selected_entries.end();
 	    ++ iter)
 	{
+		Gtk::TreePath unsorted_path =
+			m_sorted->convert_path_to_child_path(*iter);
 		Gtk::TreeIter tree_iter =
-			m_settings.get_list()->get_iter(*iter);
+			m_settings.get_list()->get_iter(unsorted_path);
 
 		LocalDocumentInfo* info =
 			(*tree_iter)[m_settings.columns.info];
@@ -213,18 +225,22 @@ void Gobby::DocumentList::on_selection_changed()
 	m_btn_subscribe.set_sensitive(false);
 }
 
-#include <iostream>
-bool Gobby::DocumentList::on_tree_button_press(GdkEventButton* event)
+void Gobby::DocumentList::on_row_activated(const Gtk::TreePath& path,
+                                           Gtk::TreeViewColumn* column)
 {
-	std::cout << event->button << std::endl;
-
-	// Double-clicking on a entry is like clicking on the subscribe
-	// button
-	if(event->type == GDK_2BUTTON_PRESS && event->button == 1)
+	if(m_buffer != NULL && m_buffer->is_open())
 	{
 		on_subscribe();
-		return true;
 	}
 
-	return false;
+	Gtk::TreePath unsorted_path = m_sorted->convert_path_to_child_path(path);
+
+	// select the tab if present
+	Gtk::TreeIter tree_iter = m_settings.get_list()->get_iter(unsorted_path);
+	LocalDocumentInfo* info = (*tree_iter)[m_settings.columns.info];
+
+	if(info->get_subscription_state() == LocalDocumentInfo::SUBSCRIBED)
+	{
+		m_folder.select_document(*info);
+	}
 }
