@@ -19,23 +19,28 @@
 #include <gtkmm/messagedialog.h>
 #include <gtkmm/stock.h>
 #include <obby/format_string.hpp>
-#include "io/buffer_wrapper.hpp"
 #include "common.hpp"
-#include "defaultdialog.hpp"
 #include "colorsel.hpp"
 #include "passworddialog.hpp"
+#include "document.hpp"
 #include "joinprogressdialog.hpp"
 
-namespace {
-	// TODO: How to use std::logical_and and such?
-	bool logical_and(bool lhs, bool rhs) { return lhs && rhs; }
-	bool logical_or(bool lhs, bool rhs) { return lhs || rhs; }
-	bool logical_not(bool arg) { return !arg; }
+namespace
+{
+	Glib::ustring make_user_password_info(const Glib::ustring& username)
+	{
+		obby::format_string str(
+			Gobby::_(
+				"User password for user '%0%' required. You "
+				"may either choose another user name, type "
+				"in your user password or cancel the "
+				"connection."
+			)
+		);
 
-	typedef bool(*compare_ustring_func)(
-		const Glib::ustring&,
-		const Glib::ustring&
-	);
+		str << username.raw();
+		return str.str();
+	}
 
 	obby::colour gdk_to_obby(const Gdk::Color& color)
 	{
@@ -59,16 +64,20 @@ namespace {
 Gobby::JoinProgressDialog::Prompt::Prompt(Gtk::Window& parent,
                                           const Glib::ustring& title,
                                           const Glib::ustring& info,
-                                          Gtk::Widget& widget,
                                           const Gtk::StockID& icon):
-	DefaultDialog(title, parent, true, true), m_table(2, 2), m_info(info),
+	Gtk::Dialog(title, parent, true, true), m_table(2, 2), m_info(info),
 	m_icon(icon, Gtk::ICON_SIZE_DIALOG)
 {
 	m_info.set_line_wrap(true);
 
 	m_table.attach(m_icon, 0, 1, 0, 2, Gtk::SHRINK, Gtk::SHRINK);
-	m_table.attach(m_info, 1, 2, 0, 1, Gtk::EXPAND | Gtk::FILL, Gtk::SHRINK);
-	m_table.attach(widget, 1, 2, 1, 2, Gtk::EXPAND | Gtk::FILL, Gtk::SHRINK);
+	m_table.attach(
+		m_info,
+		1, 2, 0, 1,
+		Gtk::EXPAND | Gtk::FILL,
+		Gtk::SHRINK
+	);
+
 	m_table.set_spacings(10);
 
 	get_vbox()->pack_start(m_table, Gtk::PACK_EXPAND_WIDGET);
@@ -78,29 +87,234 @@ Gobby::JoinProgressDialog::Prompt::Prompt(Gtk::Window& parent,
 	add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
 	add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
 
+	set_default_response(Gtk::RESPONSE_OK);
 	set_border_width(10);
-
 	set_resizable(false);
 }
 
+void Gobby::JoinProgressDialog::Prompt::set_custom_widget(Widget& widget)
+{
+	m_table.attach(
+		widget,
+		1, 2, 1, 2,
+		Gtk::EXPAND | Gtk::FILL,
+		Gtk::SHRINK
+	);
+}
+
+Gobby::JoinProgressDialog::NamePrompt::
+	NamePrompt(Gtk::Window& parent,
+	           const Glib::ustring& initial_name):
+	Prompt(
+		parent,
+		obby::login::errstring(net6::login::ERROR_NAME_IN_USE),
+		_("Name is already in use. You may choose another name or "
+		  "cancel the connection."),
+		Gtk::Stock::DIALOG_QUESTION
+	),
+	m_initial_name(initial_name),
+	m_label(_("New name:"), Gtk::ALIGN_RIGHT)
+{
+	m_entry.signal_changed().connect(
+		sigc::mem_fun(*this, &NamePrompt::on_change)
+	);
+
+	m_entry.set_text(m_initial_name);
+	m_entry.set_activates_default(true);
+
+	m_box.pack_start(m_label, Gtk::PACK_SHRINK);
+	m_box.pack_start(m_entry, Gtk::PACK_EXPAND_WIDGET);
+	m_box.set_spacing(10);
+
+	m_box.show_all();
+	set_custom_widget(m_box);
+
+	m_entry.grab_focus();
+}
+
+Glib::ustring Gobby::JoinProgressDialog::NamePrompt::get_name() const
+{
+	return m_entry.get_text();
+}
+
+void Gobby::JoinProgressDialog::NamePrompt::on_change()
+{
+	const Glib::ustring name = m_entry.get_text();
+
+	set_response_sensitive(
+		Gtk::RESPONSE_OK,
+		name != m_initial_name && !name.empty()
+	);
+}
+
+Gobby::JoinProgressDialog::ColorPrompt::
+	ColorPrompt(Gtk::Window& parent,
+	            Config::ParentEntry& config_entry,
+	            const Gdk::Color& initial_color):
+	Prompt(
+		parent,
+		obby::login::errstring(obby::login::ERROR_COLOUR_IN_USE),
+		_("Colour is already in use. You may choose another colour or "
+		  "cancel the connection."),
+		Gtk::Stock::DIALOG_QUESTION
+	),
+	m_button(config_entry)
+{
+	m_button.set_color(initial_color);
+	m_button.show_all();
+
+	set_custom_widget(m_button);
+	m_button.grab_focus();
+}
+
+Gdk::Color Gobby::JoinProgressDialog::ColorPrompt::get_color() const
+{
+	return m_button.get_color();
+}
+
+Gobby::JoinProgressDialog::SessionPasswordPrompt::
+	SessionPasswordPrompt(Gtk::Window& parent):
+	Prompt(
+		parent,
+		obby::login::errstring(
+			obby::login::ERROR_WRONG_GLOBAL_PASSWORD
+		),
+		_("Session password required. You have to type in the "
+		  "password to be able to join the obby session."),
+		Gtk::Stock::DIALOG_AUTHENTICATION
+	),
+	m_label(_("Session password:"), Gtk::ALIGN_RIGHT)
+{
+	m_entry.signal_changed().connect(
+		sigc::mem_fun(*this, &SessionPasswordPrompt::on_change)
+	);
+
+	m_entry.set_activates_default(true);
+	m_entry.set_visibility(false);
+
+	m_box.pack_start(m_label, Gtk::PACK_SHRINK);
+	m_box.pack_start(m_entry, Gtk::PACK_EXPAND_WIDGET);
+	m_box.set_spacing(10);
+
+	m_box.show_all();
+	set_custom_widget(m_box);
+	set_response_sensitive(Gtk::RESPONSE_OK, false);
+
+	m_entry.grab_focus();
+}
+
+Glib::ustring Gobby::JoinProgressDialog::SessionPasswordPrompt::
+	get_password() const
+{
+	return m_entry.get_text();
+}
+
+void Gobby::JoinProgressDialog::SessionPasswordPrompt::on_change()
+{
+	set_response_sensitive(Gtk::RESPONSE_OK, !m_entry.get_text().empty() );
+}
+
+Gobby::JoinProgressDialog::UserPasswordPrompt::
+	UserPasswordPrompt(Gtk::Window& parent,
+	                   const Glib::ustring& initial_name):
+	Prompt(
+		parent,
+		obby::login::errstring(obby::login::ERROR_WRONG_USER_PASSWORD),
+		make_user_password_info(initial_name),
+		Gtk::Stock::DIALOG_AUTHENTICATION
+	),
+	m_initial_name(initial_name), m_table(2, 2),
+	m_lbl_name(_("New name:"), Gtk::ALIGN_RIGHT),
+	m_lbl_password(_("User password:"), Gtk::ALIGN_RIGHT)
+{
+	m_ent_name.set_text(m_initial_name);
+	m_ent_password.set_visibility(false);
+
+	m_ent_name.set_activates_default(true);
+	m_ent_password.set_activates_default(true);
+
+	m_ent_name.signal_changed().connect(
+		sigc::mem_fun(*this, &UserPasswordPrompt::on_change)
+	);
+
+	m_ent_password.signal_changed().connect(
+		sigc::mem_fun(*this, &UserPasswordPrompt::on_change)
+	);
+
+	m_table.attach(
+		m_lbl_name,
+		0, 1, 0, 1,
+		Gtk::SHRINK | Gtk::FILL, Gtk::SHRINK
+	);
+
+	m_table.attach(
+		m_lbl_password,
+		0, 1, 1, 2,
+		Gtk::SHRINK | Gtk::FILL, Gtk::SHRINK
+	);
+
+	m_table.attach(
+		m_ent_name,
+		1, 2, 0, 1,
+		Gtk::EXPAND | Gtk::FILL, Gtk::SHRINK
+	);
+
+	m_table.attach(
+		m_ent_password,
+		1, 2, 1, 2,
+		Gtk::EXPAND | Gtk::FILL, Gtk::SHRINK
+	);
+
+	m_table.set_spacings(5);
+
+	set_response_sensitive(Gtk::RESPONSE_OK, false);
+	m_ent_password.grab_focus();
+
+	m_table.show_all();
+	set_custom_widget(m_table);
+
+	m_ent_password.grab_focus();
+}
+
+Glib::ustring Gobby::JoinProgressDialog::UserPasswordPrompt::get_name() const
+{
+	return m_ent_name.get_text();
+}
+
+Glib::ustring Gobby::JoinProgressDialog::UserPasswordPrompt::
+	get_password() const
+{
+	return m_ent_password.get_text();
+}
+
+void Gobby::JoinProgressDialog::UserPasswordPrompt::on_change()
+{
+	const Glib::ustring name = m_ent_name.get_name();
+	const Glib::ustring password = m_ent_password.get_name();
+
+	set_response_sensitive(
+		Gtk::RESPONSE_OK,
+		!name.empty() && (name != m_initial_name || !password.empty())
+	);
+}
+
 Gobby::JoinProgressDialog::JoinProgressDialog(Gtk::Window& parent,
-                                              Config& config,
+                                              Config::ParentEntry& config_entry,
                                               const Glib::ustring& hostname,
                                               unsigned int port,
                                               const Glib::ustring& username,
-                                              const Gdk::Color& color)
- : ProgressDialog(_("Joining obby session..."), parent), m_config(config),
-   m_hostname(hostname), m_port(port), m_username(username), m_color(color),
-   m_got_done(false), m_got_welcome(false)
+                                              const Gdk::Color& color):
+	ProgressDialog(_("Joining obby session..."), parent),
+	m_config_entry(config_entry), m_hostname(hostname), m_port(port),
+	m_username(username), m_color(color), m_got_welcome(false),
+	m_got_done(false)
 {
 	obby::format_string str("Connecting to %0%...");
 	str << hostname;
 	set_status_text(str.str() );
-
-	// TODO: Write new username and colour (if any) to config, on destruction!
 }
 
-std::auto_ptr<obby::client_buffer> Gobby::JoinProgressDialog::get_buffer()
+std::auto_ptr<Gobby::ClientBuffer> Gobby::JoinProgressDialog::get_buffer()
 {
 	return m_buffer;
 }
@@ -120,17 +334,17 @@ void Gobby::JoinProgressDialog::on_thread(Thread& thread)
 	// Dialog may be closed now
 	unlock(thread);
 
-	std::auto_ptr<obby::client_buffer> buffer; // Resulting buffer
+	std::auto_ptr<ClientBuffer> buffer; // Resulting buffer
 	Glib::ustring error; // Error message
 
 	// Establish connection
 	try
 	{
-		buffer.reset(
-			new obby::io::client_buffer(
-#ifdef WIN32
-				parent
-#endif
+		buffer.reset(new ClientBuffer);
+
+		buffer->set_document_template(
+			ClientBuffer::document_type::template_type(
+				*buffer
 			)
 		);
 
@@ -180,10 +394,14 @@ void Gobby::JoinProgressDialog::on_done()
 		display_error(m_error);
 		// Bad response
 		response(Gtk::RESPONSE_CANCEL);
+		return;
 	}
 
 	// Thread has established connection, wait for welcome packet
-	set_status_text(_("Waiting for welcome packet...") );
+	// The welcome event will be emitted as soon as the connection
+	// has been encrypted. Maybe we should have another step before
+	// this that really waits for the welcome packet.
+	set_status_text(_("Setting up connection encryption (TLS)...") );
 	set_progress_fraction(1.0/4.0);
 	m_got_done = true;
 
@@ -200,7 +418,7 @@ void Gobby::JoinProgressDialog::on_welcome()
 	m_got_welcome = true;
 
 	// Do nothing if we have not already got the done signal from the
-	// thread.
+	// thread. on_done will recall on_welcome then.
 	if(!m_got_done)
 		return;
 
@@ -256,70 +474,11 @@ void Gobby::JoinProgressDialog::on_login_failed(obby::login::error error)
 
 bool Gobby::JoinProgressDialog::on_prompt_name(connection_settings& settings)
 {
-	Gtk::HBox box;
-	Gtk::Label label(_("New name:") );
-	Gtk::Entry entry;
-	entry.set_text(m_username);
+	NamePrompt prompt(*this, m_username);
 
-	box.pack_start(label, Gtk::PACK_SHRINK);
-	box.pack_start(entry, Gtk::PACK_EXPAND_WIDGET);
-	box.set_spacing(10);
-
-	Prompt prompt(
-		*this,
-		obby::login::errstring(net6::login::ERROR_NAME_IN_USE),
-		_("Name is already in use. You may choose another name or "
-		  "cancel the connection."),
-		box,
-		Gtk::Stock::DIALOG_QUESTION
-	);
-
-	prompt.set_response_sensitive(Gtk::RESPONSE_OK, false);
-
-	// We are using the logical_or and logical_not functions defined above,
-	// but the standard library provides functors like std::logical_or, but
-	// how to use these?
-	sigc::slot<void> changed_slot(
-		sigc::compose(
-			sigc::bind<0>(
-				sigc::mem_fun(
-					prompt,
-					&DefaultDialog::set_response_sensitive
-				),
-				Gtk::RESPONSE_OK
-			),
-			sigc::compose(
-				sigc::ptr_fun(&logical_and),
-				sigc::compose(
-					sigc::bind(
-						sigc::ptr_fun(static_cast<
-							compare_ustring_func
-						>(&Glib::operator!=) ),
-						sigc::ref(m_username)
-					),
-					sigc::mem_fun(
-						entry,
-						&Gtk::Entry::get_text
-					)
-				),
-				sigc::compose(
-					sigc::ptr_fun(&logical_not),
-					sigc::compose(
-						&Glib::ustring::empty,
-						sigc::mem_fun(
-							entry,
-							&Gtk::Entry::get_text
-						)
-					)
-				)
-			)
-		)
-	);
-
-	entry.signal_changed().connect(changed_slot);
 	if(prompt.run() == Gtk::RESPONSE_OK)
 	{
-		settings.name = entry.get_text();
+		settings.name = prompt.get_name();
 		m_username = settings.name;
 		return true;
 	}
@@ -332,22 +491,11 @@ bool Gobby::JoinProgressDialog::on_prompt_name(connection_settings& settings)
 
 bool Gobby::JoinProgressDialog::on_prompt_colour(connection_settings& settings)
 {
-	ColorButton btn;
-	btn.set_color(m_color);
-
-	Prompt prompt(
-		*this,
-		obby::login::errstring(obby::login::ERROR_COLOUR_IN_USE),
-		_("Colour is already in use. You may choose another colour or "
-		  "cancel the connection."),
-		btn,
-		Gtk::Stock::DIALOG_QUESTION
-	);
-
+	ColorPrompt prompt(*this, m_config_entry, m_color);
 	if(prompt.run() == Gtk::RESPONSE_OK)
 	{
-		settings.colour = gdk_to_obby(btn.get_color() );
-		m_color = btn.get_color();
+		settings.colour = gdk_to_obby(prompt.get_color() );
+		m_color = prompt.get_color();
 		return true;
 	}
 	else
@@ -360,53 +508,10 @@ bool Gobby::JoinProgressDialog::on_prompt_colour(connection_settings& settings)
 bool Gobby::JoinProgressDialog::
 	on_prompt_global_password(connection_settings& settings)
 {
-	Gtk::HBox box;
-	Gtk::Label label(_("Session password:") );
-	Gtk::Entry entry;
-	entry.set_visibility(false);
-
-	box.pack_start(label, Gtk::PACK_SHRINK);
-	box.pack_start(entry, Gtk::PACK_EXPAND_WIDGET);
-	box.set_spacing(10);
-
-	Prompt prompt(
-		*this,
-		obby::login::errstring(
-			obby::login::ERROR_WRONG_GLOBAL_PASSWORD
-		),
-		_("Session password required. You have to type in the "
-		  "password to be able to join the obby session."),
-		box,
-		Gtk::Stock::DIALOG_AUTHENTICATION
-	);
-
-	// Do not allow OK if the entry is empty
-	prompt.set_response_sensitive(Gtk::RESPONSE_OK, false);
-	entry.signal_changed().connect(
-		sigc::compose(
-			sigc::bind<0>(
-				sigc::mem_fun(
-					prompt,
-					&DefaultDialog::set_response_sensitive
-				),
-				Gtk::RESPONSE_OK
-			),
-			sigc::compose(
-				sigc::ptr_fun(&logical_not),
-				sigc::compose(
-					&Glib::ustring::empty,
-					sigc::mem_fun(
-						entry,
-						&Gtk::Entry::get_text
-					)
-				)
-			)
-		)
-	);
-
+	SessionPasswordPrompt prompt(*this);
 	if(prompt.run() == Gtk::RESPONSE_OK)
 	{
-		settings.global_password = entry.get_text();
+		settings.global_password = prompt.get_password();
 		return true;
 	}
 	else
@@ -419,102 +524,12 @@ bool Gobby::JoinProgressDialog::
 bool Gobby::JoinProgressDialog::
 	on_prompt_user_password(connection_settings& settings)
 {
-	Gtk::Table table;
-	Gtk::Label lbl_name(_("New name:"), Gtk::ALIGN_RIGHT);
-	Gtk::Label lbl_password(_("User password:"), Gtk::ALIGN_RIGHT);
-	Gtk::Entry ent_name;
-	Gtk::Entry ent_password;
-	ent_name.set_text(m_username);
-	ent_password.set_visibility(false);
-
-	table.attach(lbl_name, 0, 1, 0, 1,
-		Gtk::SHRINK, Gtk::SHRINK);
-	table.attach(lbl_password, 0, 1, 1, 2,
-		Gtk::SHRINK, Gtk::SHRINK);
-	table.attach(ent_name, 1, 2, 0, 1,
-		Gtk::EXPAND | Gtk::FILL, Gtk::SHRINK);
-	table.attach(ent_password, 1, 2, 1, 2,
-		Gtk::EXPAND | Gtk::FILL, Gtk::SHRINK);
-	table.set_spacings(5);
-
-	obby::format_string str(
-		_("User password for user '%0%' required. You may either "
-		  "choose another user name, type in your user password or "
-		  "cancel the connection.")
-	);
-
-	str << m_username;
-	Prompt prompt(
-		*this,
-		obby::login::errstring(obby::login::ERROR_WRONG_USER_PASSWORD),
-		str.str(),
-		table,
-		Gtk::Stock::DIALOG_AUTHENTICATION
-	);
-
-	prompt.set_response_sensitive(Gtk::RESPONSE_OK, false);
-	ent_password.grab_focus();
-
-	sigc::slot<void> changed_slot(
-		sigc::compose(
-			sigc::bind<0>(
-				sigc::mem_fun(
-					prompt,
-					&DefaultDialog::set_response_sensitive
-				),
-				Gtk::RESPONSE_OK
-			),
-			sigc::compose(
-				sigc::ptr_fun(&logical_and),
-				sigc::compose(
-					sigc::ptr_fun(&logical_not),
-					sigc::compose(
-						&Glib::ustring::empty,
-						sigc::mem_fun(
-							ent_name,
-							&Gtk::Entry::
-								get_text
-						)
-					)
-				),
-				sigc::compose(
-					sigc::ptr_fun(&logical_or),
-					sigc::compose(
-						sigc::bind(
-							sigc::ptr_fun(static_cast<
-								compare_ustring_func
-							>(&Glib::operator!=) ),
-							sigc::ref(m_username)
-						),
-						sigc::mem_fun(
-							ent_name,
-							&Gtk::Entry::get_text
-						)
-					),
-					sigc::compose(
-						sigc::ptr_fun(&logical_not),
-						sigc::compose(
-							&Glib::ustring::empty,
-							sigc::mem_fun(
-								ent_password,
-								&Gtk::Entry::
-									get_text
-							)
-						)
-					)
-				)
-			)
-		)
-	);
-
-	ent_name.signal_changed().connect(changed_slot);
-	ent_password.signal_changed().connect(changed_slot);
-
+	UserPasswordPrompt prompt(*this, m_username);
 	if(prompt.run() == Gtk::RESPONSE_OK)
 	{
-		m_username = ent_name.get_text();
-		settings.name = ent_name.get_text();
-		settings.user_password = ent_password.get_text();
+		m_username = prompt.get_name();
+		settings.name = prompt.get_name();
+		settings.user_password = prompt.get_password();
 		return true;
 	}
 	else
@@ -555,6 +570,6 @@ void Gobby::JoinProgressDialog::display_error(const Glib::ustring& message)
 
 void Gobby::JoinProgressDialog::on_response(int response_id)
 {
-	m_config["session"]["name"].set(m_username);
-	m_config["session"]["color"].set(m_color);
+	m_config_entry.set_value("name", m_username);
+	m_config_entry.set_value("color", m_color);
 }
