@@ -55,8 +55,12 @@ Gobby::PreferencesDialog::Page::Page():
 	set_border_width(10);
 }
 
+#ifndef GTKMM_DISABLE_DEPRECATED
 Gobby::PreferencesDialog::Editor::Editor(const Preferences& preferences,
                                          Gtk::Tooltips& tooltips):
+#else
+Gobby::PreferencesDialog::Editor::Editor(const Preferences& preferences):
+#endif
 	m_frame_tab(_("Tab Stops") ),
 	m_frame_indentation(_("Indentation") ),
 	m_frame_homeend(_("Home/End behaviour") ),
@@ -74,10 +78,12 @@ Gobby::PreferencesDialog::Editor::Editor(const Preferences& preferences,
 	m_ent_tab_width.set_value(tab_width);
 	m_ent_tab_width.set_increments(1, 1);
 
+#ifndef GTKMM_DISABLE_DEPRECATED
 	// TODO: Improve this description
 	tooltips.set_tip(m_btn_homeend_smart,
 		_("With this option enabled, Home/End keys move to first/last "
 		  "character before going to the start/end of the line.") );
+#endif
 
 	m_box_tab_width.set_spacing(5);
 	m_box_tab_width.pack_start(m_lbl_tab_width, Gtk::PACK_SHRINK);
@@ -305,6 +311,8 @@ Gobby::PreferencesDialog::Behaviour::Behaviour(const Preferences& preferences):
 	bool auto_open = preferences.behaviour.auto_open_new_documents;
 
 	m_btn_auto_open.set_active(auto_open);
+	m_box_documents.set_spacing(5);
+	m_box_documents.set_border_width(5);
 	m_box_documents.pack_start(m_btn_auto_open, Gtk::PACK_SHRINK);
 	m_frame_documents.add(m_box_documents);
 
@@ -349,14 +357,19 @@ Gobby::PreferencesDialog::FileList::FileList(Gtk::Window& parent,
 	m_file_list(Gtk::ListStore::create(file_columns) )
 {
 #ifdef WITH_GTKSOURCEVIEW2
-	const GSList* list =
-		gtk_source_language_manager_list_languages(lang_mgr);
+	GSList* languages = NULL;
+	const gchar* const* ids = gtk_source_language_manager_get_language_ids(lang_mgr);
+	for(const gchar* const* id = ids; *id != NULL; ++ id)
+	{
+		GtkSourceLanguage* language = gtk_source_language_manager_get_language(lang_mgr, *id);
+		languages = g_slist_prepend(languages, language);
+	}
 #else
 	const GSList* list =
 		gtk_source_languages_manager_get_available_languages(lang_mgr);
+	GSList* languages = g_slist_copy(const_cast<GSList*>(list));
 #endif
 
-	GSList* languages = g_slist_copy(const_cast<GSList*>(list));
 	languages = g_slist_sort(languages, &lang_sort);
 
 	for(GSList* iter = languages; iter != NULL; iter = iter->next)
@@ -520,13 +533,13 @@ void Gobby::PreferencesDialog::FileList::
 	                   const Glib::ustring& new_text)
 {
 #ifdef WITH_GTKSOURCEVIEW2
-	const GSList* list =
-		gtk_source_language_manager_list_languages(m_lang_mgr);
+	const gchar* const* ids = gtk_source_language_manager_get_language_ids(m_lang_mgr);
 
 	GtkSourceLanguage* lang = NULL;
-	for(; list != NULL; list = list->next)
+	for(const gchar* const* id = ids; *id != NULL; ++ id)
 	{
-		GtkSourceLanguage* language = GTK_SOURCE_LANGUAGE(list->data);
+		GtkSourceLanguage* language = gtk_source_language_manager_get_language(m_lang_mgr, *id);
+
 		gchar** mime_types = gtk_source_language_get_mime_types(language);
 		for(gchar** mime_type = mime_types; *mime_type != NULL; ++ mime_type)
 		{
@@ -687,9 +700,16 @@ Gobby::PreferencesDialog::PreferencesDialog(Gtk::Window& parent,
                                             GtkSourceLanguageManager* lang_mgr,
                                             bool local)
  : Gtk::Dialog(_("Preferences"), parent, true),
-   m_page_editor(preferences, m_tooltips), m_page_view(preferences),
-   m_page_appearance(preferences), m_page_font(preferences),
-   m_page_behaviour(preferences), m_page_files(*this, preferences, lang_mgr)
+#ifndef GTKMM_DISABLE_DEPRECATED
+   m_page_editor(preferences, m_tooltips),
+#else
+   m_page_editor(preferences),
+#endif
+   m_page_view(preferences),
+   m_page_appearance(preferences),
+   m_page_font(preferences),
+   m_page_behaviour(preferences),
+   m_page_files(*this, preferences, lang_mgr)
 {
 	m_notebook.append_page(m_page_editor, _("Editor") );
 	m_notebook.append_page(m_page_view, _("View") );
